@@ -758,6 +758,41 @@ mfex_check_tcp_data_offset(const struct tcp_header *tcp)
     return ret;
 }
 
+static void
+mfex_ipv4_set_hwol(struct dp_packet *pkt)
+{
+    dp_packet_hwol_set_tx_ipv4(pkt);
+    if (dp_packet_ip_checksum_good(pkt)) {
+        dp_packet_hwol_set_tx_ip_csum(pkt);
+    }
+}
+
+static void
+mfex_ipv6_set_hwol(struct dp_packet *pkt)
+{
+    dp_packet_hwol_set_tx_ipv6(pkt);
+}
+
+static void
+mfex_tcp_set_hwol(struct dp_packet *pkt)
+{
+    dp_packet_ol_l4_csum_check_partial(pkt);
+    if (dp_packet_l4_checksum_good(pkt)
+        || dp_packet_ol_l4_csum_partial(pkt)) {
+        dp_packet_hwol_set_csum_tcp(pkt);
+    }
+}
+
+static void
+mfex_udp_set_hwol(struct dp_packet *pkt)
+{
+    dp_packet_ol_l4_csum_check_partial(pkt);
+    if (dp_packet_l4_checksum_good(pkt)
+        || dp_packet_ol_l4_csum_partial(pkt)) {
+        dp_packet_hwol_set_csum_udp(pkt);
+    }
+}
+
 /* Generic loop to process any mfex profile. This code is specialized into
  * multiple actual MFEX implementation functions. Its marked ALWAYS_INLINE
  * to ensure the compiler specializes each instance. The code is marked "hot"
@@ -859,6 +894,8 @@ mfex_avx512_process(struct dp_packet_batch *packets,
                 const struct tcp_header *tcp = (void *)&pkt[38];
                 mfex_handle_tcp_flags(tcp, &blocks[7]);
                 dp_packet_update_rss_hash_ipv4_tcp_udp(packet);
+                mfex_ipv4_set_hwol(packet);
+                mfex_tcp_set_hwol(packet);
             } break;
 
         case PROFILE_ETH_VLAN_IPV4_UDP: {
@@ -871,6 +908,8 @@ mfex_avx512_process(struct dp_packet_batch *packets,
                     continue;
                 }
                 dp_packet_update_rss_hash_ipv4_tcp_udp(packet);
+                mfex_ipv4_set_hwol(packet);
+                mfex_udp_set_hwol(packet);
             } break;
 
         case PROFILE_ETH_IPV4_TCP: {
@@ -886,6 +925,8 @@ mfex_avx512_process(struct dp_packet_batch *packets,
                     continue;
                 }
                 dp_packet_update_rss_hash_ipv4_tcp_udp(packet);
+                mfex_ipv4_set_hwol(packet);
+                mfex_tcp_set_hwol(packet);
             } break;
 
         case PROFILE_ETH_IPV4_UDP: {
@@ -897,6 +938,8 @@ mfex_avx512_process(struct dp_packet_batch *packets,
                     continue;
                 }
                 dp_packet_update_rss_hash_ipv4_tcp_udp(packet);
+                mfex_ipv4_set_hwol(packet);
+                mfex_udp_set_hwol(packet);
             } break;
 
         case PROFILE_ETH_IPV6_UDP: {
@@ -915,6 +958,8 @@ mfex_avx512_process(struct dp_packet_batch *packets,
                 /* Process UDP header. */
                 mfex_handle_ipv6_l4((void *)&pkt[54], &blocks[9]);
                 dp_packet_update_rss_hash_ipv6_tcp_udp(packet);
+                mfex_ipv6_set_hwol(packet);
+                mfex_udp_set_hwol(packet);
             } break;
 
         case PROFILE_ETH_IPV6_TCP: {
@@ -938,6 +983,8 @@ mfex_avx512_process(struct dp_packet_batch *packets,
                 }
                 mfex_handle_tcp_flags(tcp, &blocks[9]);
                 dp_packet_update_rss_hash_ipv6_tcp_udp(packet);
+                mfex_ipv6_set_hwol(packet);
+                mfex_tcp_set_hwol(packet);
             } break;
 
         case PROFILE_ETH_VLAN_IPV6_TCP: {
@@ -964,6 +1011,8 @@ mfex_avx512_process(struct dp_packet_batch *packets,
                 }
                 mfex_handle_tcp_flags(tcp, &blocks[10]);
                 dp_packet_update_rss_hash_ipv6_tcp_udp(packet);
+                mfex_ipv6_set_hwol(packet);
+                mfex_tcp_set_hwol(packet);
             } break;
 
         case PROFILE_ETH_VLAN_IPV6_UDP: {
@@ -985,6 +1034,8 @@ mfex_avx512_process(struct dp_packet_batch *packets,
                 /* Process UDP header. */
                 mfex_handle_ipv6_l4((void *)&pkt[58], &blocks[10]);
                 dp_packet_update_rss_hash_ipv6_tcp_udp(packet);
+                mfex_ipv6_set_hwol(packet);
+                mfex_udp_set_hwol(packet);
             } break;
 
         case PROFILE_ETH_IPV4_NVGRE: {
@@ -995,6 +1046,8 @@ mfex_avx512_process(struct dp_packet_batch *packets,
                     continue;
                 }
                 dp_packet_update_rss_hash_ipv4(packet);
+                mfex_ipv4_set_hwol(packet);
+                mfex_udp_set_hwol(packet);
             } break;
 
         default:
