@@ -46,10 +46,10 @@ struct ovs_action_push_tnl;
 
 /* Offload-capable (HW) netdev information */
 struct netdev_hw_info {
-    bool oor;		/* Out of Offload Resources ? */
-    atomic_bool miss_api_supported;  /* hw_miss_packet_recover() supported.*/
-    int offload_count;  /* Pending (non-offloaded) flow count */
-    int pending_count;  /* Offloaded flow count */
+    bool oor;                         /* Out of Offload Resources ? */
+    atomic_bool miss_api_supported;   /* hw_miss_packet_recover() supported.*/
+    int offload_count;                /* Offloaded flow count */
+    int pending_count;                /* Pending (non-offloaded) flow count */
     OVSRCU_TYPE(void *) offload_data; /* Offload metadata. */
 };
 
