@@ -226,8 +226,10 @@ netdev_vport_construct(struct netdev *netdev_)
         update_vxlan_global_cfg(netdev_, NULL, tnl_cfg);
     } else if (!strcmp(type, "lisp")) {
         tnl_cfg->dst_port = port ? htons(port) : htons(LISP_DST_PORT);
+        VLOG_WARN("%s: 'lisp' port type is deprecated.", name);
     } else if (!strcmp(type, "stt")) {
         tnl_cfg->dst_port = port ? htons(port) : htons(STT_DST_PORT);
+        VLOG_WARN("%s: 'stt' port type is deprecated.", name);
     } else if (!strcmp(type, "gtpu")) {
         tnl_cfg->dst_port = port ? htons(port) : htons(GTPU_DST_PORT);
     } else if (!strcmp(type, "bareudp")) {
@@ -840,7 +842,8 @@ set_tunnel_config(struct netdev *dev_, const struct smap *args, char **errp)
             }
         } else if (!strcmp(node->key, "remote_cert") ||
                    !strcmp(node->key, "remote_name") ||
-                   !strcmp(node->key, "psk")) {
+                   !strcmp(node->key, "psk") ||
+                   !strncmp(node->key, "ipsec_", strlen("ipsec_"))) {
             /* When configuring OVS for IPsec, these keys may be set in the
                tunnel port's 'options' column. 'ovs-vswitchd' does not directly
                use them, but they are read by 'ovs-monitor-ipsec'. In order to
