@@ -81,7 +81,8 @@ parse_keys(bool wc_keys, const char *in)
     ds_init(&out);
     if (wc_keys) {
         odp_flow_format(odp_key.data, odp_key.size,
-                        odp_mask.data, odp_mask.size, NULL, &out, false);
+                        odp_mask.data, odp_mask.size, NULL, &out, false,
+                        false);
     } else {
         odp_flow_key_format(odp_key.data, odp_key.size, &out);
     }
