@@ -30,11 +30,14 @@
 #    - ovs_dump_netdev_provider
 #    - ovs_dump_ovs_list <struct ovs_list *> {[<structure>] [<member>] {dump}]}
 #    - ovs_dump_packets <struct dp_packet_batch|dp_packet> [tcpdump options]
+#    - ovs_dump_cmap <struct cmap *> {[<structure>] [<member>] {dump}]}
+#    - ovs_dump_hmap <struct hmap *> <structure> <member> {dump}
 #    - ovs_dump_simap <struct simap *>
 #    - ovs_dump_smap <struct smap *>
 #    - ovs_dump_udpif_keys {<udpif_name>|<udpif_address>} {short}
 #    - ovs_show_fdb {[<bridge_name>] {dbg} {hash}}
 #    - ovs_show_upcall {dbg}
+#    - ovs_dump_conntrack_conns <struct conntrack *> {short}
 #
 #  Example:
 #    $ gdb $(which ovs-vswitchd) $(pidof ovs-vswitchd)
@@ -849,6 +852,119 @@ class CmdDumpOvsList(gdb.Command):
                         member).dereference()))
 
 
+#
+# Implements the GDB "ovs_dump_cmap" command
+#
+class CmdDumpCmap(gdb.Command):
+    """Dump all nodes of a given cmap
+    Usage:
+      ovs_dump_cmap <struct cmap *> {[<structure>] [<member>] {dump}]}
+
+    For example dump all the rules in a dpcls_subtable:
+
+    (gdb) ovs_dump_cmap &subtable->rules
+    (struct cmap *) 0x3e02758
+
+    This is not very useful, so please use this with the container_of mode:
+
+    (gdb) ovs_dump_cmap &subtable->rules "struct dpcls_rule" cmap_node
+    (struct dpcls_rule *) 0x3e02758
+
+    Now you can manually use the print command to show the content, or use the
+    dump option to dump the structure for all nodes:
+
+    (gdb) ovs_dump_cmap &subtable->rules "struct dpcls_rule" cmap_node dump
+    (struct dpcls_rule *) 0x3e02758 =
+    {cmap_node = {next = {p = 0x0}}, mask = 0x3dfe100, flow = {hash = ...
+    """
+    def __init__(self):
+        super(CmdDumpCmap, self).__init__("ovs_dump_cmap",
+                                          gdb.COMMAND_DATA)
+
+    def invoke(self, arg, from_tty):
+        arg_list = gdb.string_to_argv(arg)
+        typeobj = None
+        member = None
+        dump = False
+
+        if len(arg_list) != 1 and len(arg_list) != 3 and len(arg_list) != 4:
+            print("usage: ovs_dump_cmap <struct cmap *> "
+                  "{[<structure>] [<member>] {dump}]}")
+            return
+
+        cmap = gdb.parse_and_eval(arg_list[0]).cast(
+            gdb.lookup_type('struct cmap').pointer())
+
+        if len(arg_list) >= 3:
+            typeobj = arg_list[1]
+            member = arg_list[2]
+            if len(arg_list) == 4 and arg_list[3] == "dump":
+                dump = True
+
+        for node in ForEachCMAP(cmap.dereference()):
+            if typeobj is None or member is None:
+                print("(struct cmap *) {}".format(node))
+            else:
+                print("({} *) {} {}".format(
+                    typeobj,
+                    container_of(node,
+                                 gdb.lookup_type(typeobj).pointer(), member),
+                    "=" if dump else ""))
+                if dump:
+                    print("  {}\n".format(container_of(
+                        node,
+                        gdb.lookup_type(typeobj).pointer(),
+                        member).dereference()))
+
+
+#
+# Implements the GDB "ovs_dump_hmap" command
+#
+class CmdDumpHmap(gdb.Command):
+    """Dump all nodes of a given hmap
+    Usage:
+      ovs_dump_hmap <struct hmap *> <structure> <member> {dump}
+
+    For example dump all the bridges when the all_bridges variable is
+    optimized out due to LTO:
+
+    (gdb) ovs_dump_hmap "&'all_bridges.lto_priv.0'" "struct bridge" "node"
+    (struct bridge *) 0x55ec43069c70
+    (struct bridge *) 0x55ec430428a0
+    (struct bridge *) 0x55ec430a55f0
+
+    The 'dump' option will also include the full structure content in the
+    output.
+    """
+    def __init__(self):
+        super(CmdDumpHmap, self).__init__("ovs_dump_hmap",
+                                          gdb.COMMAND_DATA)
+
+    def invoke(self, arg, from_tty):
+        arg_list = gdb.string_to_argv(arg)
+        typeobj = None
+        member = None
+        dump = False
+
+        if len(arg_list) != 3 and len(arg_list) != 4:
+            print("usage: ovs_dump_hmap <struct hmap *> "
+                  "<structure> <member> {dump}")
+            return
+
+        hmap = gdb.parse_and_eval(arg_list[0]).cast(
+            gdb.lookup_type('struct hmap').pointer())
+
+        typeobj = arg_list[1]
+        member = arg_list[2]
+        if len(arg_list) == 4 and arg_list[3] == "dump":
+            dump = True
+
+        for node in ForEachHMAP(hmap.dereference(), typeobj, member):
+            print("({} *) {} {}".format(typeobj, node, "=" if dump else ""))
+            if dump:
+                print("  {}\n".format(node.dereference()))
+
+
 #
 # Implements the GDB "ovs_dump_simap" command
 #
@@ -1391,7 +1507,8 @@ class CmdDumpPackets(gdb.Command):
             print("Error, unsupported argument type: {}".format(str(val.type)))
             return
 
-        tcpdump(pkt_list, args=tcpdump_args)
+        stdout = tcpdump(pkt_list, args=tcpdump_args, getfd=True, quiet=True)
+        gdb.write(stdout.read().decode("utf8", "replace"))
 
     def extract_pkt(self, pkt):
         pkt_fields = pkt.type.keys()
@@ -1434,6 +1551,69 @@ class CmdDumpPackets(gdb.Command):
         return packet
 
 
+#
+# Implements the GDB "ovs_dump_conntrack_conns" command
+#
+class CmdDumpDpConntrackConn(gdb.Command):
+    """Dump all connections in a conntrack set
+    Usage:
+      ovs_dump_conntrack_conns <struct conntrack *> {short}
+
+      <struct conntrack *> : Pointer to conntrack
+      short                : Only dump conn structure addresses,
+                             no content details
+
+    Example dumping all <struct conn> connections:
+
+    (gdb) ovs_dump_conntrack_conns 0x5606339c25e0
+    (struct conn *) 0x7f32c000a8c0: expiration = ... nw_proto = 1
+    (struct conn *) 0x7f32c00489d0: expiration = ... nw_proto = 6
+    (struct conn *) 0x7f32c0153bb0: expiration = ... nw_proto = 17
+
+    (gdb) ovs_dump_conntrack_conns 0x5606339c25e0 short
+    (struct conn *) 0x7f32c000a8c0
+    (struct conn *) 0x7f32c00489d0
+    (struct conn *) 0x7f32c0153bb0
+    """
+    def __init__(self):
+        super(CmdDumpDpConntrackConn, self).__init__(
+            "ovs_dump_conntrack_conns",
+            gdb.COMMAND_DATA)
+
+    @staticmethod
+    def display_single_conn(conn, dir_, indent=0, short=False):
+        indent = " " * indent
+        if short:
+            print("{}(struct conn *) {}".format(indent, conn))
+        else:
+            print("{}(struct conn *) {}: expiration = {}, mark = {}, "
+                  "dl_type = {}, zone = {}, nw_proto = {}".format(
+                      indent, conn, conn['expiration'],
+                      conn['mark'], conn['key_node'][dir_]['key']['dl_type'],
+                      conn['key_node'][dir_]['key']['zone'],
+                      conn['key_node'][dir_]['key']['nw_proto']))
+
+    def invoke(self, arg, from_tty):
+        arg_list = gdb.string_to_argv(arg)
+        if len(arg_list) not in (1, 2) or \
+           (len(arg_list) == 2 and arg_list[1] != "short"):
+            print("usage: ovs_dump_conntrack_conns <struct conntrack *> "
+                  "{short}")
+            return
+
+        ct = gdb.parse_and_eval(arg_list[0]).cast(
+            gdb.lookup_type('struct conntrack').pointer())
+
+        for key_node in ForEachCMAP(ct["conns"],
+                                    "struct conn_key_node", "cm_node"):
+            node = container_of(
+                key_node,
+                gdb.lookup_type('struct conn').pointer(),
+                "key_node")
+            self.display_single_conn(node, key_node['dir'],
+                                     short="short" in arg_list[1:])
+
+
 #
 # Initialize all GDB commands
 #
@@ -1448,8 +1628,11 @@ CmdDumpNetdevProvider()
 CmdDumpOfpacts()
 CmdDumpOvsList()
 CmdDumpPackets()
+CmdDumpCmap()
+CmdDumpHmap()
 CmdDumpSimap()
 CmdDumpSmap()
 CmdDumpUdpifKeys()
 CmdShowFDB()
 CmdShowUpcall()
+CmdDumpDpConntrackConn()
